#!/bin/bash
# Copyright (c) 2019 Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl.

set -o errexit
set -o nounset
set -o pipefail
set -o errtrace

# variables
REGISTRY=container-registry.oracle.com/olcne
K8S_VERSION=v1.14.9-2
FLANNEL_VERSION=v0.10.0-1
DASHBOARD_VERSION=v1.10.1-1
ETCD_VERSION=3.3.10-1
PAUSE_VERSION=3.1
COREDNS_VERSION=1.3.1
ARCH=""
USELOCAL=0

function ops {
  local ops="${1:-}"
  local registry="${2:-${REGISTRY}}"

  for image in "${IMAGES[@]}"; do
  	docker "${ops}" "${registry}/${image}"
  done
}

function tag {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"

  for image in "${IMAGES[@]}"; do
	docker tag "${from}/${image}" "${to}/${image}"
  done
}

function updatelist {
        IMAGENAMELIST="flannel:${FLANNEL_VERSION}${ARCH},\
etcd${ARCH}:${ETCD_VERSION},\
pause${ARCH}:${PAUSE_VERSION},\
kube-controller-manager${ARCH}:${K8S_VERSION},\
kube-scheduler${ARCH}:${K8S_VERSION},\
kube-apiserver${ARCH}:${K8S_VERSION},\
kube-proxy${ARCH}:${K8S_VERSION},\
kubernetes-dashboard:${DASHBOARD_VERSION},\
coredns:${COREDNS_VERSION}"
        IFS=',' read -d '' -ra IMAGES < <(printf '%s,\0' "$IMAGENAMELIST")
}

function check {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"
  local version="${3:-}"
  local image="pause"

  # start docker-engine
  echo "Checking if docker is active ..."
  if ! systemctl -q is-active docker; then
        echo "[ERROR] docker is not started ... please start docker"
        exit 1
  fi

  version=${version//v/}
  if  [ "${version}" = "1.13.10" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0
        ETCD_VERSION=3.2.24
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
	COREDNS_VERSION=1.2.6
	updatelist
  elif  [ "${version}" = "1.14.9" ]; then
        K8S_VERSION=${K8S_VERSION}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.3.10-1
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.3.1
        updatelist
  elif  [ "v${version}" = "${K8S_VERSION}" ]; then
	:
  else
        echo "[ERROR] ${version} is not a valid version"
        exit 1
  fi

  echo "Checking for pull access from ${from} registry ..."
  # shellcheck disable=SC2155
  local result=$(echo ""|docker pull "${from}/${image}${ARCH}:${PAUSE_VERSION}" 2>&1|grep "Please login")
  if [ -n "${result}" ]; then
        echo "[ERROR] Please login with valid credential to the ${from}"
        echo "        # docker login ${from}"
        exit 1
  else
        if ! docker pull "${from}/${image}${ARCH}:${PAUSE_VERSION}"; then
                echo "[ERROR] docker cannot pull ${image}:${PAUSE_VERSION} from ${from} registry"
                exit 1
        fi
  fi

  docker tag "${from}/${image}${ARCH}:${PAUSE_VERSION}" "${to}/${image}${ARCH}:${PAUSE_VERSION}"

  echo "Checking for push access to ${to} registry ..."
  if ! docker push "${to}/${image}${ARCH}:${PAUSE_VERSION}"; then
	echo "[ERROR] docker cannot push ${image}${ARCH}:${PAUSE_VERSION} to ${to} registry"
	exit 1
  fi
}

function usage {
	echo "This script is to help pulling docker images from default container-registry.oracle.com to a local repo" >&2
	echo "usage: " >&2
	echo "  $0 --to registry [--local --from registry --image-names <list of image:tag names> --version ${K8S_VERSION}]" >&2
	exit 1
}

# MAIN
ARG=($@)
PASS="0"
VERSION="${K8S_VERSION}"
FROM="${REGISTRY}"
# update the list with defaults
updatelist
for((i=0; i<$#; i++)); do
	if [ "${ARG[${i}]}" = "--to" ]; then
		PASS="1"
		i=$((i+1))
		TO="${ARG[${i}]:-}"
		if [ -z "${TO}" ]; then
			echo "[ERROR] Please provide a valid local registry location"
			exit 1
		fi
	elif [ "${ARG[${i}]}" = "--from" ]; then
		i=$((i+1))
		FROM="${ARG[${i}]:-${REGISTRY}}"
	elif [ "${ARG[${i}]}" = "--version" ]; then
		i=$((i+1))
		VERSION="${ARG[${i}]:-"${K8S_VERSION}"}"
	elif [ "${ARG[${i}]}" = "--local" ]; then
		i=$((i+1))
		USELOCAL=1
	elif [ "${ARG[${i}]}" = "--image-names" ]; then
		i=$((i+1))
		IFS=',' read -d '' -ra IMAGES < <(printf '%s,\0' "${ARG[${i}]:-}")
		if [ -z "${IMAGES}" ]; then
			echo "[ERROR] Please provide a list of image:tag names"
			exit
		fi
	else
		usage
	fi
done

if [ "${PASS}" != "1" ]; then
	usage
fi

if [ "$USELOCAL" -eq 1 ]; then
        VERSION="${K8S_VERSION}"
        if ls /usr/local/share/kubeadm/*.tar &> /dev/null; then
                find /usr/local/share/kubeadm/*.tar -print0 | xargs -0 | xargs -n1 docker load -i
        fi
        if ls /usr/local/share/olcne/*.tar &> /dev/null; then
                find /usr/local/share/olcne/*.tar -print0 | xargs -0 | xargs -n1 docker load -i
        fi
else
	echo "Pulling images [${VERSION}] from: [${FROM}] to: [${TO}]"
	check "${FROM}" "${TO}" "${VERSION}"
	ops "pull" "${FROM}" # pull from registry
fi

tag "${FROM}" "${TO}"
ops "push" "${TO}" # push to registry
echo "[SUCCESS] All images pushed to [${TO}]"
